@@ -25,7 +25,6 @@ import (
 	"time"
 
 	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
 	v1 "k8s.io/api/core/v1"
 	discovery "k8s.io/api/discovery/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
@@ -36,17 +35,14 @@ import (
 	"k8s.io/apimachinery/pkg/util/intstr"
 	"k8s.io/apimachinery/pkg/util/rand"
 	"k8s.io/apimachinery/pkg/util/wait"
-	utilfeature "k8s.io/apiserver/pkg/util/feature"
 	"k8s.io/client-go/informers"
 	"k8s.io/client-go/kubernetes/fake"
 	k8stesting "k8s.io/client-go/testing"
 	"k8s.io/client-go/tools/cache"
-	featuregatetesting "k8s.io/component-base/featuregate/testing"
 	"k8s.io/kubernetes/pkg/controller"
 	"k8s.io/kubernetes/pkg/controller/endpointslice/topologycache"
 	endpointutil "k8s.io/kubernetes/pkg/controller/util/endpoint"
 	endpointsliceutil "k8s.io/kubernetes/pkg/controller/util/endpointslice"
-	"k8s.io/kubernetes/pkg/features"
 	utilpointer "k8s.io/utils/pointer"
 )
 
@@ -419,12 +415,11 @@ func TestSyncService(t *testing.T) {
 	deletionTimestamp := metav1.Now()
 
 	testcases := []struct {
-		name                   string
-		service                *v1.Service
-		pods                   []*v1.Pod
-		expectedEndpointPorts  []discovery.EndpointPort
-		expectedEndpoints      []discovery.Endpoint
-		terminatingGateEnabled bool
+		name                  string
+		service               *v1.Service
+		pods                  []*v1.Pod
+		expectedEndpointPorts []discovery.EndpointPort
+		expectedEndpoints     []discovery.Endpoint
 	}{
 		{
 			name: "pods with multiple IPs and Service with ipFamilies=ipv4",
@@ -523,7 +518,9 @@ func TestSyncService(t *testing.T) {
 			expectedEndpoints: []discovery.Endpoint{
 				{
 					Conditions: discovery.EndpointConditions{
-						Ready: utilpointer.BoolPtr(true),
+						Ready:       utilpointer.BoolPtr(true),
+						Serving:     utilpointer.BoolPtr(true),
+						Terminating: utilpointer.BoolPtr(false),
 					},
 					Addresses: []string{"10.0.0.1"},
 					TargetRef: &v1.ObjectReference{Kind: "Pod", Namespace: "default", Name: "pod0"},
@@ -531,7 +528,9 @@ func TestSyncService(t *testing.T) {
 				},
 				{
 					Conditions: discovery.EndpointConditions{
-						Ready: utilpointer.BoolPtr(true),
+						Ready:       utilpointer.BoolPtr(true),
+						Serving:     utilpointer.BoolPtr(true),
+						Terminating: utilpointer.BoolPtr(false),
 					},
 					Addresses: []string{"10.0.0.2"},
 					TargetRef: &v1.ObjectReference{Kind: "Pod", Namespace: "default", Name: "pod1"},
@@ -636,7 +635,9 @@ func TestSyncService(t *testing.T) {
 			expectedEndpoints: []discovery.Endpoint{
 				{
 					Conditions: discovery.EndpointConditions{
-						Ready: utilpointer.BoolPtr(true),
+						Ready:       utilpointer.BoolPtr(true),
+						Serving:     utilpointer.BoolPtr(true),
+						Terminating: utilpointer.BoolPtr(false),
 					},
 					Addresses: []string{"fd08::5678:0000:0000:9abc:def0"},
 					TargetRef: &v1.ObjectReference{Kind: "Pod", Namespace: "default", Name: "pod1"},
@@ -645,7 +646,7 @@ func TestSyncService(t *testing.T) {
 			},
 		},
 		{
-			name: "Terminating pods with EndpointSliceTerminatingCondition enabled",
+			name: "Terminating pods",
 			service: &v1.Service{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:              "foobar",
@@ -758,114 +759,9 @@ func TestSyncService(t *testing.T) {
 					NodeName:  utilpointer.StringPtr("node-1"),
 				},
 			},
-			terminatingGateEnabled: true,
 		},
 		{
-			name: "Terminating pods with EndpointSliceTerminatingCondition disabled",
-			service: &v1.Service{
-				ObjectMeta: metav1.ObjectMeta{
-					Name:              "foobar",
-					Namespace:         "default",
-					CreationTimestamp: creationTimestamp,
-				},
-				Spec: v1.ServiceSpec{
-					Ports: []v1.ServicePort{
-						{Name: "tcp-example", TargetPort: intstr.FromInt(80), Protocol: v1.ProtocolTCP},
-						{Name: "udp-example", TargetPort: intstr.FromInt(161), Protocol: v1.ProtocolUDP},
-						{Name: "sctp-example", TargetPort: intstr.FromInt(3456), Protocol: v1.ProtocolSCTP},
-					},
-					Selector:   map[string]string{"foo": "bar"},
-					IPFamilies: []v1.IPFamily{v1.IPv4Protocol},
-				},
-			},
-			pods: []*v1.Pod{
-				{
-					// one ready pod for comparison
-					ObjectMeta: metav1.ObjectMeta{
-						Namespace:         "default",
-						Name:              "pod0",
-						Labels:            map[string]string{"foo": "bar"},
-						DeletionTimestamp: nil,
-					},
-					Spec: v1.PodSpec{
-						Containers: []v1.Container{{
-							Name: "container-1",
-						}},
-						NodeName: "node-1",
-					},
-					Status: v1.PodStatus{
-						PodIP: "10.0.0.1",
-						PodIPs: []v1.PodIP{{
-							IP: "10.0.0.1",
-						}},
-						Conditions: []v1.PodCondition{
-							{
-								Type:   v1.PodReady,
-								Status: v1.ConditionTrue,
-							},
-						},
-					},
-				},
-				{
-					ObjectMeta: metav1.ObjectMeta{
-						Namespace:         "default",
-						Name:              "pod1",
-						Labels:            map[string]string{"foo": "bar"},
-						DeletionTimestamp: &deletionTimestamp,
-					},
-					Spec: v1.PodSpec{
-						Containers: []v1.Container{{
-							Name: "container-1",
-						}},
-						NodeName: "node-1",
-					},
-					Status: v1.PodStatus{
-						PodIP: "10.0.0.2",
-						PodIPs: []v1.PodIP{
-							{
-								IP: "10.0.0.2",
-							},
-						},
-						Conditions: []v1.PodCondition{
-							{
-								Type:   v1.PodReady,
-								Status: v1.ConditionTrue,
-							},
-						},
-					},
-				},
-			},
-			expectedEndpointPorts: []discovery.EndpointPort{
-				{
-					Name:     utilpointer.StringPtr("sctp-example"),
-					Protocol: protoPtr(v1.ProtocolSCTP),
-					Port:     utilpointer.Int32Ptr(int32(3456)),
-				},
-				{
-					Name:     utilpointer.StringPtr("udp-example"),
-					Protocol: protoPtr(v1.ProtocolUDP),
-					Port:     utilpointer.Int32Ptr(int32(161)),
-				},
-				{
-					Name:     utilpointer.StringPtr("tcp-example"),
-					Protocol: protoPtr(v1.ProtocolTCP),
-					Port:     utilpointer.Int32Ptr(int32(80)),
-				},
-			},
-			expectedEndpoints: []discovery.Endpoint{
-				{
-					Conditions: discovery.EndpointConditions{
-						Ready: utilpointer.BoolPtr(true),
-					},
-					Addresses: []string{"10.0.0.1"},
-					TargetRef: &v1.ObjectReference{Kind: "Pod", Namespace: "default", Name: "pod0"},
-					NodeName:  utilpointer.StringPtr("node-1"),
-				},
-			},
-			terminatingGateEnabled: false,
-		},
-		{
-			name: "Not ready terminating pods with EndpointSliceTerminatingCondition enabled",
+			name: "Not ready terminating pods",
 			service: &v1.Service{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:              "foobar",
@@ -978,118 +874,11 @@ func TestSyncService(t *testing.T) {
 					NodeName:  utilpointer.StringPtr("node-1"),
 				},
 			},
-			terminatingGateEnabled: true,
-		},
-		{
-			name: "Not ready terminating pods with EndpointSliceTerminatingCondition disabled",
-			service: &v1.Service{
-				ObjectMeta: metav1.ObjectMeta{
-					Name:              "foobar",
-					Namespace:         "default",
-					CreationTimestamp: creationTimestamp,
-				},
-				Spec: v1.ServiceSpec{
-					Ports: []v1.ServicePort{
-						{Name: "tcp-example", TargetPort: intstr.FromInt(80), Protocol: v1.ProtocolTCP},
-						{Name: "udp-example", TargetPort: intstr.FromInt(161), Protocol: v1.ProtocolUDP},
-						{Name: "sctp-example", TargetPort: intstr.FromInt(3456), Protocol: v1.ProtocolSCTP},
-					},
-					Selector:   map[string]string{"foo": "bar"},
-					IPFamilies: []v1.IPFamily{v1.IPv4Protocol},
-				},
-			},
-			pods: []*v1.Pod{
-				{
-					// one ready pod for comparison
-					ObjectMeta: metav1.ObjectMeta{
-						Namespace:         "default",
-						Name:              "pod0",
-						Labels:            map[string]string{"foo": "bar"},
-						DeletionTimestamp: nil,
-					},
-					Spec: v1.PodSpec{
-						Containers: []v1.Container{{
-							Name: "container-1",
-						}},
-						NodeName: "node-1",
-					},
-					Status: v1.PodStatus{
-						PodIP: "10.0.0.1",
-						PodIPs: []v1.PodIP{{
-							IP: "10.0.0.1",
-						}},
-						Conditions: []v1.PodCondition{
-							{
-								Type:   v1.PodReady,
-								Status: v1.ConditionTrue,
-							},
-						},
-					},
-				},
-				{
-					ObjectMeta: metav1.ObjectMeta{
-						Namespace:         "default",
-						Name:              "pod1",
-						Labels:            map[string]string{"foo": "bar"},
-						DeletionTimestamp: &deletionTimestamp,
-					},
-					Spec: v1.PodSpec{
-						Containers: []v1.Container{{
-							Name: "container-1",
-						}},
-						NodeName: "node-1",
-					},
-					Status: v1.PodStatus{
-						PodIP: "10.0.0.2",
-						PodIPs: []v1.PodIP{
-							{
-								IP: "10.0.0.2",
-							},
-						},
-						Conditions: []v1.PodCondition{
-							{
-								Type:   v1.PodReady,
-								Status: v1.ConditionFalse,
-							},
-						},
-					},
-				},
-			},
-			expectedEndpointPorts: []discovery.EndpointPort{
-				{
-					Name:     utilpointer.StringPtr("sctp-example"),
-					Protocol: protoPtr(v1.ProtocolSCTP),
-					Port:     utilpointer.Int32Ptr(int32(3456)),
-				},
-				{
-					Name:     utilpointer.StringPtr("udp-example"),
-					Protocol: protoPtr(v1.ProtocolUDP),
-					Port:     utilpointer.Int32Ptr(int32(161)),
-				},
-				{
-					Name:     utilpointer.StringPtr("tcp-example"),
-					Protocol: protoPtr(v1.ProtocolTCP),
-					Port:     utilpointer.Int32Ptr(int32(80)),
-				},
-			},
-			expectedEndpoints: []discovery.Endpoint{
-				{
-					Conditions: discovery.EndpointConditions{
-						Ready: utilpointer.BoolPtr(true),
-					},
-					Addresses: []string{"10.0.0.1"},
-					TargetRef: &v1.ObjectReference{Kind: "Pod", Namespace: "default", Name: "pod0"},
-					NodeName:  utilpointer.StringPtr("node-1"),
-				},
-			},
-			terminatingGateEnabled: false,
 		},
 	}
 
 	for _, testcase := range testcases {
 		t.Run(testcase.name, func(t *testing.T) {
-			defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.EndpointSliceTerminatingCondition, testcase.terminatingGateEnabled)()
-
 			client, esController := newController([]string{"node-1"}, time.Duration(0))
 
 			for _, pod := range testcase.pods {
@@ -1754,106 +1543,6 @@ func Test_checkNodeTopologyDistribution(t *testing.T) {
 	}
 }
 
-func TestUpdateNode(t *testing.T) {
-	nodeReadyStatus := v1.NodeStatus{
-		Allocatable: map[v1.ResourceName]resource.Quantity{
-			v1.ResourceCPU: resource.MustParse("100m"),
-		},
-		Conditions: []v1.NodeCondition{
-			{
-				Type:   v1.NodeReady,
-				Status: v1.ConditionTrue,
-			},
-		},
-	}
-	_, esController := newController(nil, time.Duration(0))
-	sliceInfo := &topologycache.SliceInfo{
-		ServiceKey:  "ns/svc",
-		AddressType: discovery.AddressTypeIPv4,
-		ToCreate: []*discovery.EndpointSlice{
-			{
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      "svc-abc",
-					Namespace: "ns",
-					Labels: map[string]string{
-						discovery.LabelServiceName: "svc",
-						discovery.LabelManagedBy:   controllerName,
-					},
-				},
-				Endpoints: []discovery.Endpoint{
-					{
-						Addresses:  []string{"172.18.0.2"},
-						Zone:       utilpointer.String("zone-a"),
-						Conditions: discovery.EndpointConditions{Ready: utilpointer.Bool(true)},
-					},
-					{
-						Addresses:  []string{"172.18.1.2"},
-						Zone:       utilpointer.String("zone-b"),
-						Conditions: discovery.EndpointConditions{Ready: utilpointer.Bool(true)},
-					},
-				},
-				AddressType: discovery.AddressTypeIPv4,
-			},
-		},
-	}
-	node1 := &v1.Node{
-		ObjectMeta: metav1.ObjectMeta{Name: "node-1"},
-		Status:     nodeReadyStatus,
-	}
-	node2 := &v1.Node{
-		ObjectMeta: metav1.ObjectMeta{Name: "node-2"},
-		Status:     nodeReadyStatus,
-	}
-	esController.nodeStore.Add(node1)
-	esController.nodeStore.Add(node2)
-	esController.addNode(node1)
-	esController.addNode(node2)
-	// The Nodes don't have the zone label, AddHints should fail (hints should not be added to Endpoints).
-	slicesToCreate, _ := esController.topologyCache.AddHints(sliceInfo)
-	expectedEndpoints := []discovery.Endpoint{
-		{
-			Addresses:  []string{"172.18.0.2"},
-			Zone:       utilpointer.String("zone-a"),
-			Conditions: discovery.EndpointConditions{Ready: utilpointer.Bool(true)},
-		},
-		{
-			Addresses:  []string{"172.18.1.2"},
-			Zone:       utilpointer.String("zone-b"),
-			Conditions: discovery.EndpointConditions{Ready: utilpointer.Bool(true)},
-		},
-	}
-	require.Len(t, slicesToCreate, 1)
-	assert.Equal(t, expectedEndpoints, slicesToCreate[0].Endpoints)
-
-	updateNode1 := node1.DeepCopy()
-	updateNode1.Labels = map[string]string{v1.LabelTopologyZone: "zone-a"}
-	updateNode2 := node2.DeepCopy()
-	updateNode2.Labels = map[string]string{v1.LabelTopologyZone: "zone-b"}
-
-	// After adding the zone label to the Nodes and calling the event handler updateNode, AddHints should succeed.
-	esController.nodeStore.Update(updateNode1)
-	esController.nodeStore.Update(updateNode2)
-	esController.updateNode(node1, updateNode1)
-	esController.updateNode(node2, updateNode2)
-	slicesToCreate, _ = esController.topologyCache.AddHints(sliceInfo)
-	expectedEndpoints = []discovery.Endpoint{
-		{
-			Addresses:  []string{"172.18.0.2"},
-			Zone:       utilpointer.String("zone-a"),
-			Conditions: discovery.EndpointConditions{Ready: utilpointer.Bool(true)},
-			Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-a"}}},
-		},
-		{
-			Addresses:  []string{"172.18.1.2"},
-			Zone:       utilpointer.String("zone-b"),
-			Conditions: discovery.EndpointConditions{Ready: utilpointer.Bool(true)},
-			Hints:      &discovery.EndpointHints{ForZones: []discovery.ForZone{{Name: "zone-b"}}},
-		},
-	}
-	require.Len(t, slicesToCreate, 1)
-	assert.Equal(t, expectedEndpoints, slicesToCreate[0].Endpoints)
-}
-
 // Test helpers
 func addPods(t *testing.T, esController *endpointSliceController, namespace string, podsCount int) {
 	t.Helper()
