@@ -53,9 +53,9 @@ type orderedIntervals []interval
 // the indexes that succeeded since the last sync.
 func calculateSucceededIndexes(job *batch.Job, pods []*v1.Pod) (orderedIntervals, orderedIntervals) {
 	var prevIntervals orderedIntervals
-	withFinalizers := trackingUncountedPods(job)
+	withFinalizers := hasJobTrackingAnnotation(job)
 	if withFinalizers {
-		prevIntervals = succeededIndexesFromString(job.Status.CompletedIndexes, int(*job.Spec.Completions))
+		prevIntervals = succeededIndexesFromJob(job)
 	}
 	newSucceeded := sets.NewInt()
 	for _, p := range pods {
@@ -152,19 +152,20 @@ func (oi orderedIntervals) has(ix int) bool {
 	return oi[hi].First <= ix
 }
 
-func succeededIndexesFromString(completedIndexes string, completions int) orderedIntervals {
-	if completedIndexes == "" {
+func succeededIndexesFromJob(job *batch.Job) orderedIntervals {
+	if job.Status.CompletedIndexes == "" {
 		return nil
 	}
 	var result orderedIntervals
 	var lastInterval *interval
-	for _, intervalStr := range strings.Split(completedIndexes, ",") {
+	completions := int(*job.Spec.Completions)
+	for _, intervalStr := range strings.Split(job.Status.CompletedIndexes, ",") {
 		limitsStr := strings.Split(intervalStr, "-")
 		var inter interval
 		var err error
 		inter.First, err = strconv.Atoi(limitsStr[0])
 		if err != nil {
-			klog.InfoS("Corrupted completed indexes interval, ignoring", "interval", intervalStr, "err", err)
+			klog.InfoS("Corrupted completed indexes interval, ignoring", "job", klog.KObj(job), "interval", intervalStr, "err", err)
 			continue
 		}
 		if inter.First >= completions {
@@ -173,7 +174,7 @@ func succeededIndexesFromString(completedIndexes string, completions int) ordere
 		if len(limitsStr) > 1 {
 			inter.Last, err = strconv.Atoi(limitsStr[1])
 			if err != nil {
-				klog.InfoS("Corrupted completed indexes interval, ignoring", "interval", intervalStr, "err", err)
+				klog.InfoS("Corrupted completed indexes interval, ignoring", "job", klog.KObj(job), "interval", intervalStr, "err", err)
 				continue
 			}
 			if inter.Last >= completions {
