@@ -19,6 +19,9 @@ package storage
 import (
 	"context"
 	"errors"
+	"fmt"
+	"reflect"
+	"strings"
 	"testing"
 
 	policyv1 "k8s.io/api/policy/v1"
@@ -31,21 +34,25 @@ import (
 	"k8s.io/apimachinery/pkg/watch"
 	genericapirequest "k8s.io/apiserver/pkg/endpoints/request"
 	"k8s.io/apiserver/pkg/registry/rest"
+	utilfeature "k8s.io/apiserver/pkg/util/feature"
 	"k8s.io/client-go/kubernetes/fake"
+	featuregatetesting "k8s.io/component-base/featuregate/testing"
 	podapi "k8s.io/kubernetes/pkg/api/pod"
 	api "k8s.io/kubernetes/pkg/apis/core"
 	"k8s.io/kubernetes/pkg/apis/policy"
+	"k8s.io/kubernetes/pkg/features"
 )
 
 func TestEviction(t *testing.T) {
 	testcases := []struct {
 		name     string
 		pdbs     []runtime.Object
+		policies []*policyv1.UnhealthyPodEvictionPolicyType
 		eviction *policy.Eviction
 
 		badNameInURL bool
 
-		expectError   bool
+		expectError   string
 		expectDeleted bool
 		podPhase      api.PodPhase
 		podName       string
@@ -58,9 +65,10 @@ func TestEviction(t *testing.T) {
 				Status:     policyv1.PodDisruptionBudgetStatus{DisruptionsAllowed: 0},
 			}},
 			eviction:    &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t1", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError: true,
+			expectError: "Cannot evict pod as it would violate the pod's disruption budget.: TooManyRequests: The disruption budget foo needs 0 healthy pods and has 0 currently",
 			podPhase:    api.PodRunning,
 			podName:     "t1",
+			policies:    []*policyv1.UnhealthyPodEvictionPolicyType{nil, unhealthyPolicyPtr(policyv1.IfHealthyBudget)}, // AlwaysAllow would terminate the pod since Running pods are not guarded by this policy
 		},
 		{
 			name: "matching pdbs with no disruptions allowed, pod pending",
@@ -70,7 +78,7 @@ func TestEviction(t *testing.T) {
 				Status:     policyv1.PodDisruptionBudgetStatus{DisruptionsAllowed: 0},
 			}},
 			eviction:      &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t2", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError:   false,
+			expectError:   "",
 			podPhase:      api.PodPending,
 			expectDeleted: true,
 			podName:       "t2",
@@ -83,7 +91,7 @@ func TestEviction(t *testing.T) {
 				Status:     policyv1.PodDisruptionBudgetStatus{DisruptionsAllowed: 0},
 			}},
 			eviction:      &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t3", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError:   false,
+			expectError:   "",
 			podPhase:      api.PodSucceeded,
 			expectDeleted: true,
 			podName:       "t3",
@@ -96,7 +104,7 @@ func TestEviction(t *testing.T) {
 				Status:     policyv1.PodDisruptionBudgetStatus{DisruptionsAllowed: 0},
 			}},
 			eviction:      &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t4", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError:   false,
+			expectError:   "",
 			podPhase:      api.PodFailed,
 			expectDeleted: true,
 			podName:       "t4",
@@ -132,91 +140,109 @@ func TestEviction(t *testing.T) {
 			}},
 			badNameInURL: true,
 			eviction:     &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t7", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError:  true,
+			expectError:  "name in URL does not match name in Eviction object: BadRequest",
 			podName:      "t7",
 		},
 	}
 
-	for _, tc := range testcases {
-		t.Run(tc.name, func(t *testing.T) {
-			testContext := genericapirequest.WithNamespace(genericapirequest.NewContext(), metav1.NamespaceDefault)
-			storage, _, statusStorage, server := newStorage(t)
-			defer server.Terminate(t)
-			defer storage.Store.DestroyFunc()
-
-			pod := validNewPod()
-			pod.Name = tc.podName
-			pod.Labels = map[string]string{"a": "true"}
-			pod.Spec.NodeName = "foo"
-			if _, err := storage.Create(testContext, pod, nil, &metav1.CreateOptions{}); err != nil {
-				t.Error(err)
+	for _, unhealthyPodEvictionPolicy := range []*policyv1.UnhealthyPodEvictionPolicyType{nil, unhealthyPolicyPtr(policyv1.IfHealthyBudget), unhealthyPolicyPtr(policyv1.AlwaysAllow)} {
+		for _, tc := range testcases {
+			if len(tc.policies) > 0 && !hasUnhealthyPolicy(tc.policies, unhealthyPodEvictionPolicy) {
+				// unhealthyPodEvictionPolicy is not covered by this test
+				continue
 			}
+			t.Run(fmt.Sprintf("%v with %v policy", tc.name, unhealthyPolicyStr(unhealthyPodEvictionPolicy)), func(t *testing.T) {
+				defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.PDBUnhealthyPodEvictionPolicy, true)()
 
-			if tc.podPhase != "" {
-				pod.Status.Phase = tc.podPhase
-				_, _, err := statusStorage.Update(testContext, pod.Name, rest.DefaultUpdatedObjectInfo(pod), rest.ValidateAllObjectFunc, rest.ValidateAllObjectUpdateFunc, false, &metav1.UpdateOptions{})
-				if err != nil {
-					t.Errorf("Unexpected error: %v", err)
+				// same test runs multiple times, make copy of objects to have unique ones
+				evictionCopy := tc.eviction.DeepCopy()
+				var pdbsCopy []runtime.Object
+				for _, pdb := range tc.pdbs {
+					pdbCopy := pdb.DeepCopyObject()
+					pdbCopy.(*policyv1.PodDisruptionBudget).Spec.UnhealthyPodEvictionPolicy = unhealthyPodEvictionPolicy
+					pdbsCopy = append(pdbsCopy, pdbCopy)
 				}
-			}
 
-			client := fake.NewSimpleClientset(tc.pdbs...)
-			evictionRest := newEvictionStorage(storage.Store, client.PolicyV1())
+				testContext := genericapirequest.WithNamespace(genericapirequest.NewContext(), metav1.NamespaceDefault)
+				storage, _, statusStorage, server := newStorage(t)
+				defer server.Terminate(t)
+				defer storage.Store.DestroyFunc()
 
-			name := pod.Name
-			if tc.badNameInURL {
-				name += "bad-name"
-			}
+				pod := validNewPod()
+				pod.Name = tc.podName
+				pod.Labels = map[string]string{"a": "true"}
+				pod.Spec.NodeName = "foo"
+				if _, err := storage.Create(testContext, pod, nil, &metav1.CreateOptions{}); err != nil {
+					t.Error(err)
+				}
 
-			_, err := evictionRest.Create(testContext, name, tc.eviction, nil, &metav1.CreateOptions{})
-			//_, err = evictionRest.Create(testContext, name, tc.eviction, nil, &metav1.CreateOptions{})
-			if (err != nil) != tc.expectError {
-				t.Errorf("expected error=%v, got %v; name %v", tc.expectError, err, pod.Name)
-				return
-			}
-			if tc.badNameInURL {
-				if err == nil {
-					t.Error("expected error here, but got nil")
+				if tc.podPhase != "" {
+					pod.Status.Phase = tc.podPhase
+					_, _, err := statusStorage.Update(testContext, pod.Name, rest.DefaultUpdatedObjectInfo(pod), rest.ValidateAllObjectFunc, rest.ValidateAllObjectUpdateFunc, false, &metav1.UpdateOptions{})
+					if err != nil {
+						t.Errorf("Unexpected error: %v", err)
+					}
+				}
+
+				client := fake.NewSimpleClientset(pdbsCopy...)
+				evictionRest := newEvictionStorage(storage.Store, client.PolicyV1())
+
+				name := pod.Name
+				if tc.badNameInURL {
+					name += "bad-name"
+				}
+
+				_, err := evictionRest.Create(testContext, name, evictionCopy, nil, &metav1.CreateOptions{})
+				gotErr := errToString(err)
+				if gotErr != tc.expectError {
+					t.Errorf("error mismatch: expected %v, got %v; name %v", tc.expectError, gotErr, pod.Name)
 					return
 				}
-				if err.Error() != "name in URL does not match name in Eviction object" {
-					t.Errorf("got unexpected error: %v", err)
+				if tc.badNameInURL {
+					if err == nil {
+						t.Error("expected error here, but got nil")
+						return
+					}
+					if err.Error() != "name in URL does not match name in Eviction object" {
+						t.Errorf("got unexpected error: %v", err)
+					}
 				}
-			}
-			if tc.expectError {
-				return
-			}
-
-			existingPod, err := storage.Get(testContext, pod.Name, &metav1.GetOptions{})
-			if tc.expectDeleted {
-				if !apierrors.IsNotFound(err) {
-					t.Errorf("expected to be deleted, lookup returned %#v", existingPod)
+				if tc.expectError != "" {
+					return
 				}
-				return
-			} else if apierrors.IsNotFound(err) {
-				t.Errorf("expected graceful deletion, got %v", err)
-				return
-			}
 
-			if err != nil {
-				t.Errorf("%#v", err)
-				return
-			}
+				existingPod, err := storage.Get(testContext, pod.Name, &metav1.GetOptions{})
+				if tc.expectDeleted {
+					if !apierrors.IsNotFound(err) {
+						t.Errorf("expected to be deleted, lookup returned %#v", existingPod)
+					}
+					return
+				} else if apierrors.IsNotFound(err) {
+					t.Errorf("expected graceful deletion, got %v", err)
+					return
+				}
 
-			if existingPod.(*api.Pod).DeletionTimestamp == nil {
-				t.Errorf("expected gracefully deleted pod with deletionTimestamp set, got %#v", existingPod)
-			}
-		})
+				if err != nil {
+					t.Errorf("%#v", err)
+					return
+				}
+
+				if existingPod.(*api.Pod).DeletionTimestamp == nil {
+					t.Errorf("expected gracefully deleted pod with deletionTimestamp set, got %#v", existingPod)
+				}
+			})
+		}
 	}
 }
 
-func TestEvictionIngorePDB(t *testing.T) {
+func TestEvictionIgnorePDB(t *testing.T) {
 	testcases := []struct {
 		name     string
 		pdbs     []runtime.Object
+		policies []*policyv1.UnhealthyPodEvictionPolicyType
 		eviction *policy.Eviction
 
-		expectError         bool
+		expectError         string
 		podPhase            api.PodPhase
 		podName             string
 		expectedDeleteCount int
@@ -231,12 +257,12 @@ func TestEvictionIngorePDB(t *testing.T) {
 				Status:     policyv1.PodDisruptionBudgetStatus{DisruptionsAllowed: 0},
 			}},
 			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t1", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError:         false,
+			expectError:         "",
 			podPhase:            api.PodPending,
 			podName:             "t1",
 			expectedDeleteCount: 3,
 		},
-		// This test case is critical.  If it is removed or broken we may
+		// This test case is critical. If it is removed or broken we may
 		// regress and allow a pod to be deleted without checking PDBs when the
 		// pod should not be deleted.
 		{
@@ -247,10 +273,30 @@ func TestEvictionIngorePDB(t *testing.T) {
 				Status:     policyv1.PodDisruptionBudgetStatus{DisruptionsAllowed: 0},
 			}},
 			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t2", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError:         true,
+			expectError:         "Cannot evict pod as it would violate the pod's disruption budget.: TooManyRequests: The disruption budget foo needs 0 healthy pods and has 0 currently",
 			podPhase:            api.PodPending,
 			podName:             "t2",
 			expectedDeleteCount: 1,
+			policies:            []*policyv1.UnhealthyPodEvictionPolicyType{nil, unhealthyPolicyPtr(policyv1.IfHealthyBudget)}, // AlwaysAllow does not continueToPDBs, but straight to deletion
+		},
+		{
+			name: "pdbs No disruptions allowed, pod pending, first delete conflict, pod becomes running, skip PDB check, conflict",
+			pdbs: []runtime.Object{&policyv1.PodDisruptionBudget{
+				ObjectMeta: metav1.ObjectMeta{Name: "foo", Namespace: "default"},
+				Spec:       policyv1.PodDisruptionBudgetSpec{Selector: &metav1.LabelSelector{MatchLabels: map[string]string{"a": "true"}}},
+				Status:     policyv1.PodDisruptionBudgetStatus{DisruptionsAllowed: 0},
+			}},
+			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t2", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
+			expectError:         "Cannot evict pod as it would violate the pod's disruption budget.: TooManyRequests: The disruption budget foo is still being processed by the server.",
+			podPhase:            api.PodPending,
+			podName:             "t2",
+			podTerminating:      false,
+			expectedDeleteCount: 2,
+			prc: &api.PodCondition{
+				Type:   api.PodReady,
+				Status: api.ConditionFalse,
+			},
+			policies: []*policyv1.UnhealthyPodEvictionPolicyType{unhealthyPolicyPtr(policyv1.AlwaysAllow)}, // nil, IfHealthyBudget continueToPDBs
 		},
 		{
 			name: "pdbs disruptions allowed, pod pending, first delete conflict, pod becomes running, continueToPDBs",
@@ -260,10 +306,11 @@ func TestEvictionIngorePDB(t *testing.T) {
 				Status:     policyv1.PodDisruptionBudgetStatus{DisruptionsAllowed: 1},
 			}},
 			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t3", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError:         false,
+			expectError:         "",
 			podPhase:            api.PodPending,
 			podName:             "t3",
 			expectedDeleteCount: 2,
+			policies:            []*policyv1.UnhealthyPodEvictionPolicyType{nil, unhealthyPolicyPtr(policyv1.IfHealthyBudget)}, // AlwaysAllow does not continueToPDBs, but straight to deletion if pod not healthy (ready)
 		},
 		{
 			name: "pod pending, always conflict on delete",
@@ -273,7 +320,7 @@ func TestEvictionIngorePDB(t *testing.T) {
 				Status:     policyv1.PodDisruptionBudgetStatus{DisruptionsAllowed: 0},
 			}},
 			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t4", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError:         true,
+			expectError:         `Operation cannot be fulfilled on tests "2": message: Conflict`,
 			podPhase:            api.PodPending,
 			podName:             "t4",
 			expectedDeleteCount: EvictionsRetry.Steps,
@@ -286,7 +333,7 @@ func TestEvictionIngorePDB(t *testing.T) {
 				Status:     policyv1.PodDisruptionBudgetStatus{DisruptionsAllowed: 0},
 			}},
 			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t5", Namespace: "default"}, DeleteOptions: metav1.NewRVDeletionPrecondition("userProvided")},
-			expectError:         true,
+			expectError:         `Operation cannot be fulfilled on tests "2": message: Conflict`,
 			podPhase:            api.PodPending,
 			podName:             "t5",
 			expectedDeleteCount: 1,
@@ -299,7 +346,7 @@ func TestEvictionIngorePDB(t *testing.T) {
 				Status:     policyv1.PodDisruptionBudgetStatus{DisruptionsAllowed: 0},
 			}},
 			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t6", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(300)},
-			expectError:         false,
+			expectError:         "",
 			podName:             "t6",
 			expectedDeleteCount: 1,
 			podTerminating:      true,
@@ -310,14 +357,14 @@ func TestEvictionIngorePDB(t *testing.T) {
 				ObjectMeta: metav1.ObjectMeta{Name: "foo", Namespace: "default"},
 				Spec:       policyv1.PodDisruptionBudgetSpec{Selector: &metav1.LabelSelector{MatchLabels: map[string]string{"a": "true"}}},
 				Status: policyv1.PodDisruptionBudgetStatus{
-					// This simulates 3 pods desired, our pod healthy, unhealthy pod is not ours.
+					// This simulates 3 pods expected, our pod healthy, unhealthy pod is not ours.
 					DisruptionsAllowed: 0,
 					CurrentHealthy:     2,
 					DesiredHealthy:     2,
 				},
 			}},
 			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t7", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError:         true,
+			expectError:         "Cannot evict pod as it would violate the pod's disruption budget.: TooManyRequests: The disruption budget foo needs 2 healthy pods and has 2 currently",
 			podName:             "t7",
 			expectedDeleteCount: 0,
 			podTerminating:      false,
@@ -327,20 +374,42 @@ func TestEvictionIngorePDB(t *testing.T) {
 				Status: api.ConditionTrue,
 			},
 		},
+		{
+			name: "matching pdbs with disruptions allowed, pod running, pod healthy, healthy pod ours, deletes pod by honoring the PDB",
+			pdbs: []runtime.Object{&policyv1.PodDisruptionBudget{
+				ObjectMeta: metav1.ObjectMeta{Name: "foo", Namespace: "default"},
+				Spec:       policyv1.PodDisruptionBudgetSpec{Selector: &metav1.LabelSelector{MatchLabels: map[string]string{"a": "true"}}},
+				Status: policyv1.PodDisruptionBudgetStatus{
+					DisruptionsAllowed: 1,
+					CurrentHealthy:     3,
+					DesiredHealthy:     3,
+				},
+			}},
+			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t8", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
+			expectError:         "",
+			podName:             "t8",
+			expectedDeleteCount: 1,
+			podTerminating:      false,
+			podPhase:            api.PodRunning,
+			prc: &api.PodCondition{
+				Type:   api.PodReady,
+				Status: api.ConditionTrue,
+			},
+		},
 		{
 			name: "matching pdbs with no disruptions allowed, pod running, pod unhealthy, unhealthy pod ours",
 			pdbs: []runtime.Object{&policyv1.PodDisruptionBudget{
 				ObjectMeta: metav1.ObjectMeta{Name: "foo", Namespace: "default"},
 				Spec:       policyv1.PodDisruptionBudgetSpec{Selector: &metav1.LabelSelector{MatchLabels: map[string]string{"a": "true"}}},
 				Status: policyv1.PodDisruptionBudgetStatus{
-					// This simulates 3 pods desired, our pod unhealthy
+					// This simulates 3 pods expected, our pod unhealthy
 					DisruptionsAllowed: 0,
 					CurrentHealthy:     2,
 					DesiredHealthy:     2,
 				},
 			}},
 			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t8", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError:         false,
+			expectError:         "",
 			podName:             "t8",
 			expectedDeleteCount: 1,
 			podTerminating:      false,
@@ -350,6 +419,25 @@ func TestEvictionIngorePDB(t *testing.T) {
 				Status: api.ConditionFalse,
 			},
 		},
+		{
+			name: "matching pdbs with no disruptions allowed, pod running, pod unhealthy, unhealthy pod ours, skips the PDB check and deletes",
+			pdbs: []runtime.Object{&policyv1.PodDisruptionBudget{
+				ObjectMeta: metav1.ObjectMeta{Name: "foo", Namespace: "default"},
+				Spec:       policyv1.PodDisruptionBudgetSpec{Selector: &metav1.LabelSelector{MatchLabels: map[string]string{"a": "true"}}},
+				Status:     policyv1.PodDisruptionBudgetStatus{DisruptionsAllowed: 0},
+			}},
+			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t8", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
+			expectError:         "",
+			podName:             "t8",
+			expectedDeleteCount: 1,
+			podTerminating:      false,
+			podPhase:            api.PodRunning,
+			prc: &api.PodCondition{
+				Type:   api.PodReady,
+				Status: api.ConditionFalse,
+			},
+			policies: []*policyv1.UnhealthyPodEvictionPolicyType{unhealthyPolicyPtr(policyv1.AlwaysAllow)}, // nil, IfHealthyBudget would not skip the PDB check
+		},
 		{
 			// This case should return the 529 retry error.
 			name: "matching pdbs with no disruptions allowed, pod running, pod unhealthy, unhealthy pod ours, resource version conflict",
@@ -357,14 +445,14 @@ func TestEvictionIngorePDB(t *testing.T) {
 				ObjectMeta: metav1.ObjectMeta{Name: "foo", Namespace: "default"},
 				Spec:       policyv1.PodDisruptionBudgetSpec{Selector: &metav1.LabelSelector{MatchLabels: map[string]string{"a": "true"}}},
 				Status: policyv1.PodDisruptionBudgetStatus{
-					// This simulates 3 pods desired, our pod unhealthy
+					// This simulates 3 pods expected, our pod unhealthy
 					DisruptionsAllowed: 0,
 					CurrentHealthy:     2,
 					DesiredHealthy:     2,
 				},
 			}},
 			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t9", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError:         true,
+			expectError:         "Cannot evict pod as it would violate the pod's disruption budget.: TooManyRequests: The disruption budget foo is still being processed by the server.",
 			podName:             "t9",
 			expectedDeleteCount: 1,
 			podTerminating:      false,
@@ -381,14 +469,14 @@ func TestEvictionIngorePDB(t *testing.T) {
 				ObjectMeta: metav1.ObjectMeta{Name: "foo", Namespace: "default"},
 				Spec:       policyv1.PodDisruptionBudgetSpec{Selector: &metav1.LabelSelector{MatchLabels: map[string]string{"a": "true"}}},
 				Status: policyv1.PodDisruptionBudgetStatus{
-					// This simulates 3 pods desired, our pod unhealthy
+					// This simulates 3 pods expected, our pod unhealthy
 					DisruptionsAllowed: 0,
 					CurrentHealthy:     2,
 					DesiredHealthy:     2,
 				},
 			}},
 			eviction:            &policy.Eviction{ObjectMeta: metav1.ObjectMeta{Name: "t10", Namespace: "default"}, DeleteOptions: metav1.NewDeleteOptions(0)},
-			expectError:         true,
+			expectError:         "test designed to error: BadRequest",
 			podName:             "t10",
 			expectedDeleteCount: 1,
 			podTerminating:      false,
@@ -400,50 +488,68 @@ func TestEvictionIngorePDB(t *testing.T) {
 		},
 	}
 
-	for _, tc := range testcases {
-		t.Run(tc.name, func(t *testing.T) {
-			testContext := genericapirequest.WithNamespace(genericapirequest.NewContext(), metav1.NamespaceDefault)
-			ms := &mockStore{
-				deleteCount: 0,
+	for _, unhealthyPodEvictionPolicy := range []*policyv1.UnhealthyPodEvictionPolicyType{unhealthyPolicyPtr(policyv1.AlwaysAllow), nil, unhealthyPolicyPtr(policyv1.IfHealthyBudget)} {
+		for _, tc := range testcases {
+			if len(tc.policies) > 0 && !hasUnhealthyPolicy(tc.policies, unhealthyPodEvictionPolicy) {
+				// unhealthyPodEvictionPolicy is not covered by this test
+				continue
 			}
+			t.Run(fmt.Sprintf("%v with %v policy", tc.name, unhealthyPolicyStr(unhealthyPodEvictionPolicy)), func(t *testing.T) {
+				defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.PDBUnhealthyPodEvictionPolicy, true)()
 
-			pod := validNewPod()
-			pod.Name = tc.podName
-			pod.Labels = map[string]string{"a": "true"}
-			pod.Spec.NodeName = "foo"
-			if tc.podPhase != "" {
-				pod.Status.Phase = tc.podPhase
-			}
-
-			if tc.podTerminating {
-				currentTime := metav1.Now()
-				pod.ObjectMeta.DeletionTimestamp = &currentTime
-			}
-
-			// Setup pod condition
-			if tc.prc != nil {
-				if !podapi.UpdatePodCondition(&pod.Status, tc.prc) {
-					t.Fatalf("Unable to update pod ready condition")
+				// same test runs 3 times, make copy of objects to have unique ones
+				evictionCopy := tc.eviction.DeepCopy()
+				prcCopy := tc.prc.DeepCopy()
+				var pdbsCopy []runtime.Object
+				for _, pdb := range tc.pdbs {
+					pdbCopy := pdb.DeepCopyObject()
+					pdbCopy.(*policyv1.PodDisruptionBudget).Spec.UnhealthyPodEvictionPolicy = unhealthyPodEvictionPolicy
+					pdbsCopy = append(pdbsCopy, pdbCopy)
 				}
-			}
 
-			client := fake.NewSimpleClientset(tc.pdbs...)
-			evictionRest := newEvictionStorage(ms, client.PolicyV1())
+				testContext := genericapirequest.WithNamespace(genericapirequest.NewContext(), metav1.NamespaceDefault)
+				ms := &mockStore{
+					deleteCount: 0,
+				}
 
-			name := pod.Name
-			ms.pod = pod
+				pod := validNewPod()
+				pod.Name = tc.podName
+				pod.Labels = map[string]string{"a": "true"}
+				pod.Spec.NodeName = "foo"
+				if tc.podPhase != "" {
+					pod.Status.Phase = tc.podPhase
+				}
 
-			_, err := evictionRest.Create(testContext, name, tc.eviction, nil, &metav1.CreateOptions{})
-			if (err != nil) != tc.expectError {
-				t.Errorf("expected error=%v, got %v; name %v", tc.expectError, err, pod.Name)
-				return
-			}
+				if tc.podTerminating {
+					currentTime := metav1.Now()
+					pod.ObjectMeta.DeletionTimestamp = &currentTime
+				}
 
-			if tc.expectedDeleteCount != ms.deleteCount {
-				t.Errorf("expected delete count=%v, got %v; name %v", tc.expectedDeleteCount, ms.deleteCount, pod.Name)
-			}
+				// Setup pod condition
+				if tc.prc != nil {
+					if !podapi.UpdatePodCondition(&pod.Status, prcCopy) {
+						t.Fatalf("Unable to update pod ready condition")
+					}
+				}
 
-		})
+				client := fake.NewSimpleClientset(pdbsCopy...)
+				evictionRest := newEvictionStorage(ms, client.PolicyV1())
+
+				name := pod.Name
+				ms.pod = pod
+
+				_, err := evictionRest.Create(testContext, name, evictionCopy, nil, &metav1.CreateOptions{})
+				gotErr := errToString(err)
+				if gotErr != tc.expectError {
+					t.Errorf("error mismatch: expected %v, got %v; name %v", tc.expectError, gotErr, pod.Name)
+					return
+				}
+
+				if tc.expectedDeleteCount != ms.deleteCount {
+					t.Errorf("expected delete count=%v, got %v; name %v", tc.expectedDeleteCount, ms.deleteCount, pod.Name)
+				}
+			})
+		}
 	}
 }
 
@@ -692,3 +798,41 @@ func (ms *mockStore) ConvertToTable(ctx context.Context, object runtime.Object,
 
 func (ms *mockStore) Destroy() {
 }
+
+func unhealthyPolicyPtr(unhealthyPodEvictionPolicy policyv1.UnhealthyPodEvictionPolicyType) *policyv1.UnhealthyPodEvictionPolicyType {
+	return &unhealthyPodEvictionPolicy
+}
+
+func unhealthyPolicyStr(unhealthyPodEvictionPolicy *policyv1.UnhealthyPodEvictionPolicyType) string {
+	if unhealthyPodEvictionPolicy == nil {
+		return "nil"
+	}
+	return string(*unhealthyPodEvictionPolicy)
+}
+
+func hasUnhealthyPolicy(unhealthyPolicies []*policyv1.UnhealthyPodEvictionPolicyType, unhealthyPolicy *policyv1.UnhealthyPodEvictionPolicyType) bool {
+	for _, p := range unhealthyPolicies {
+		if reflect.DeepEqual(unhealthyPolicy, p) {
+			return true
+		}
+	}
+	return false
+}
+
+func errToString(err error) string {
+	result := ""
+	if err != nil {
+		result = err.Error()
+		if statusErr, ok := err.(*apierrors.StatusError); ok {
+			result += fmt.Sprintf(": %v", statusErr.ErrStatus.Reason)
+			if statusErr.ErrStatus.Details != nil {
+				for _, cause := range statusErr.ErrStatus.Details.Causes {
+					if !strings.HasSuffix(err.Error(), cause.Message) {
+						result += fmt.Sprintf(": %v", cause.Message)
+					}
+				}
+			}
+		}
+	}
+	return result
+}
