@@ -35,7 +35,7 @@ import (
 	fakeframework "k8s.io/kubernetes/pkg/scheduler/framework/fake"
 	st "k8s.io/kubernetes/pkg/scheduler/testing"
 	volumeutil "k8s.io/kubernetes/pkg/volume/util"
-	utilpointer "k8s.io/utils/pointer"
+	"k8s.io/utils/pointer"
 )
 
 const (
@@ -304,16 +304,6 @@ func TestCSILimits(t *testing.T) {
 			test:             "should count in-tree volumes if migration is enabled",
 			wantStatus:       framework.NewStatus(framework.Unschedulable, ErrReasonMaxVolumeCountExceeded),
 		},
-		{
-			newPod:           inTreeInlineVolPod,
-			existingPods:     []*v1.Pod{inTreeTwoVolPod},
-			filterName:       "csi",
-			maxVols:          2,
-			driverNames:      []string{csilibplugins.AWSEBSInTreePluginName, ebsCSIDriverName},
-			migrationEnabled: true,
-			limitSource:      "node",
-			test:             "nil csi node",
-		},
 		{
 			newPod:           pendingVolumePod,
 			existingPods:     []*v1.Pod{inTreeTwoVolPod},
@@ -550,8 +540,8 @@ func getFakeCSIPVLister(volumeName string, driverNames ...string) fakeframework.
 			}
 			pvLister = append(pvLister, pv)
 		}
-	}
 
+	}
 	return pvLister
 }
 
@@ -608,11 +598,10 @@ func getFakeCSIStorageClassLister(scName, provisionerName string) fakeframework.
 }
 
 func getFakeCSINodeLister(csiNode *storagev1.CSINode) fakeframework.CSINodeLister {
-	csiNodeLister := fakeframework.CSINodeLister{}
 	if csiNode != nil {
-		csiNodeLister = append(csiNodeLister, *csiNode.DeepCopy())
+		return fakeframework.CSINodeLister(*csiNode)
 	}
-	return csiNodeLister
+	return fakeframework.CSINodeLister{}
 }
 
 func getNodeWithPodAndVolumeLimits(limitSource string, pods []*v1.Pod, limit int64, driverNames ...string) (*framework.NodeInfo, *storagev1.CSINode) {
@@ -633,7 +622,7 @@ func getNodeWithPodAndVolumeLimits(limitSource string, pods []*v1.Pod, limit int
 
 	initCSINode := func() {
 		csiNode = &storagev1.CSINode{
-			ObjectMeta: metav1.ObjectMeta{Name: "node-for-max-pd-test-1"},
+			ObjectMeta: metav1.ObjectMeta{Name: "csi-node-for-max-pd-test-1"},
 			Spec: storagev1.CSINodeSpec{
 				Drivers: []storagev1.CSINodeDriver{},
 			},
@@ -649,7 +638,7 @@ func getNodeWithPodAndVolumeLimits(limitSource string, pods []*v1.Pod, limit int
 			}
 			if addLimits {
 				driver.Allocatable = &storagev1.VolumeNodeResources{
-					Count: utilpointer.Int32Ptr(int32(limit)),
+					Count: pointer.Int32(int32(limit)),
 				}
 			}
 			csiNode.Spec.Drivers = append(csiNode.Spec.Drivers, driver)
