@@ -21,18 +21,14 @@ import (
 	"fmt"
 	"net"
 	"path/filepath"
-	"strings"
 	"sync"
 	"testing"
 	"time"
 
 	authenticationv1 "k8s.io/api/authentication/v1"
 	v1 "k8s.io/api/core/v1"
-	storagev1 "k8s.io/api/storage/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/types"
-	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/client-go/informers"
 	clientset "k8s.io/client-go/kubernetes"
@@ -40,7 +36,6 @@ import (
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/tools/record"
 	cloudprovider "k8s.io/cloud-provider"
-	csilibplugins "k8s.io/csi-translation-lib/plugins"
 	proxyutil "k8s.io/kubernetes/pkg/proxy/util"
 	. "k8s.io/kubernetes/pkg/volume"
 	"k8s.io/kubernetes/pkg/volume/util/hostutil"
@@ -123,6 +118,10 @@ func (f *fakeVolumeHost) GetPodsDir() string {
 	return filepath.Join(f.rootDir, "pods")
 }
 
+func (f *fakeVolumeHost) GetHostIDsForPod(pod *v1.Pod, containerUID, containerGID *int64) (hostUID, hostGID *int64, err error) {
+	return containerUID, containerGID, nil
+}
+
 func (f *fakeVolumeHost) GetPodVolumeDir(podUID types.UID, pluginName, volumeName string) string {
 	return filepath.Join(f.rootDir, "pods", string(podUID), "volumes", pluginName, volumeName)
 }
@@ -292,28 +291,8 @@ func newFakeAttachDetachVolumeHost(t *testing.T, rootDir string, kubeClient clie
 }
 
 func (f *fakeAttachDetachVolumeHost) CSINodeLister() storagelistersv1.CSINodeLister {
-	csiNode := &storagev1.CSINode{
-		ObjectMeta: metav1.ObjectMeta{Name: f.nodeName},
-		Spec: storagev1.CSINodeSpec{
-			Drivers: []storagev1.CSINodeDriver{},
-		},
-	}
-	enableMigrationOnNode(csiNode, csilibplugins.GCEPDInTreePluginName)
-	return getFakeCSINodeLister(csiNode)
-}
-
-func enableMigrationOnNode(csiNode *storagev1.CSINode, pluginName string) {
-	nodeInfoAnnotations := csiNode.GetAnnotations()
-	if nodeInfoAnnotations == nil {
-		nodeInfoAnnotations = map[string]string{}
-	}
-
-	newAnnotationSet := sets.NewString()
-	newAnnotationSet.Insert(pluginName)
-	nas := strings.Join(newAnnotationSet.List(), ",")
-	nodeInfoAnnotations[v1.MigratedPluginsAnnotationKey] = nas
-
-	csiNode.Annotations = nodeInfoAnnotations
+	// not needed for testing
+	return nil
 }
 
 func (f *fakeAttachDetachVolumeHost) CSIDriverLister() storagelistersv1.CSIDriverLister {
@@ -386,31 +365,6 @@ func (f *fakeKubeletVolumeHost) WithNode(node *v1.Node) *fakeKubeletVolumeHost {
 	return f
 }
 
-type CSINodeLister []storagev1.CSINode
-
-// Get returns a fake CSINode object.
-func (n CSINodeLister) Get(name string) (*storagev1.CSINode, error) {
-	for _, cn := range n {
-		if cn.Name == name {
-			return &cn, nil
-		}
-	}
-	return nil, fmt.Errorf("csiNode %q not found", name)
-}
-
-// List lists all CSINodes in the indexer.
-func (n CSINodeLister) List(selector labels.Selector) (ret []*storagev1.CSINode, err error) {
-	return nil, fmt.Errorf("not implemented")
-}
-
-func getFakeCSINodeLister(csiNode *storagev1.CSINode) CSINodeLister {
-	csiNodeLister := CSINodeLister{}
-	if csiNode != nil {
-		csiNodeLister = append(csiNodeLister, *csiNode.DeepCopy())
-	}
-	return csiNodeLister
-}
-
 func (f *fakeKubeletVolumeHost) SetKubeletError(err error) {
 	f.mux.Lock()
 	defer f.mux.Unlock()
