@@ -20,7 +20,6 @@ import (
 	"context"
 	"fmt"
 	"net"
-	"reflect"
 	"sync"
 	"time"
 
@@ -43,14 +42,13 @@ import (
 	clientretry "k8s.io/client-go/util/retry"
 	cloudprovider "k8s.io/cloud-provider"
 	controllersmetrics "k8s.io/component-base/metrics/prometheus/controllers"
-	"k8s.io/component-base/metrics/prometheus/ratelimiter"
 	nodeutil "k8s.io/component-helpers/node/util"
 )
 
 const (
-	// Maximal number of concurrent route operation API calls.
+	// Maximal number of concurrent CreateRoute API calls.
 	// TODO: This should be per-provider.
-	maxConcurrentRouteOperations int = 200
+	maxConcurrentRouteCreations int = 200
 )
 
 var updateNetworkConditionBackoff = wait.Backoff{
@@ -71,10 +69,6 @@ type RouteController struct {
 }
 
 func New(routes cloudprovider.Routes, kubeClient clientset.Interface, nodeInformer coreinformers.NodeInformer, clusterName string, clusterCIDRs []*net.IPNet) *RouteController {
-	if kubeClient != nil && kubeClient.CoreV1().RESTClient().GetRateLimiter() != nil {
-		ratelimiter.RegisterMetricAndTrackRateLimiterUsage("route_controller", kubeClient.CoreV1().RESTClient().GetRateLimiter())
-	}
-
 	if len(clusterCIDRs) == 0 {
 		klog.Fatal("RouteController: Must specify clusterCIDR.")
 	}
@@ -141,56 +135,22 @@ func (rc *RouteController) reconcileNodeRoutes(ctx context.Context) error {
 	return rc.reconcile(ctx, nodes, routeList)
 }
 
-type routeAction string
-
-var (
-	keep   routeAction = "keep"
-	add    routeAction = "add"
-	remove routeAction = "remove"
-	update routeAction = "update"
-)
-
-type routeNode struct {
-	name            types.NodeName
-	addrs           []v1.NodeAddress
-	routes          []*cloudprovider.Route
-	cidrWithActions *map[string]routeAction
-}
-
 func (rc *RouteController) reconcile(ctx context.Context, nodes []*v1.Node, routes []*cloudprovider.Route) error {
 	var l sync.Mutex
-	// routeMap includes info about a target Node and its addresses, routes and a map between Pod CIDRs and actions.
-	// If action is add/remove, the route will be added/removed.
-	// If action is keep, the route will not be touched.
-	// If action is update, the route will be deleted and then added.
-	routeMap := make(map[types.NodeName]routeNode)
-
-	// Put current routes into routeMap.
+	// for each node a map of podCIDRs and their created status
+	nodeRoutesStatuses := make(map[types.NodeName]map[string]bool)
+	// routeMap maps routeTargetNode->route
+	routeMap := make(map[types.NodeName][]*cloudprovider.Route)
 	for _, route := range routes {
-		if route.TargetNode == "" {
-			continue
+		if route.TargetNode != "" {
+			routeMap[route.TargetNode] = append(routeMap[route.TargetNode], route)
 		}
-		rn, ok := routeMap[route.TargetNode]
-		if !ok {
-			rn = routeNode{
-				name:            route.TargetNode,
-				addrs:           []v1.NodeAddress{},
-				routes:          []*cloudprovider.Route{},
-				cidrWithActions: &map[string]routeAction{},
-			}
-		} else if rn.routes == nil {
-			rn.routes = []*cloudprovider.Route{}
-		}
-		rn.routes = append(rn.routes, route)
-		routeMap[route.TargetNode] = rn
 	}
 
 	wg := sync.WaitGroup{}
-	rateLimiter := make(chan struct{}, maxConcurrentRouteOperations)
+	rateLimiter := make(chan struct{}, maxConcurrentRouteCreations)
 	// searches existing routes by node for a matching route
 
-	// Check Nodes and their Pod CIDRs. Then put expected route actions into nodePodCIDRActionMap.
-	// Add addresses of Nodes into routeMap.
 	for _, node := range nodes {
 		// Skip if the node hasn't been assigned a CIDR yet.
 		if len(node.Spec.PodCIDRs) == 0 {
@@ -198,101 +158,26 @@ func (rc *RouteController) reconcile(ctx context.Context, nodes []*v1.Node, rout
 		}
 		nodeName := types.NodeName(node.Name)
 		l.Lock()
-		rn, ok := routeMap[nodeName]
-		if !ok {
-			rn = routeNode{
-				name:            nodeName,
-				addrs:           []v1.NodeAddress{},
-				routes:          []*cloudprovider.Route{},
-				cidrWithActions: &map[string]routeAction{},
-			}
-		}
-		rn.addrs = node.Status.Addresses
-		routeMap[nodeName] = rn
+		nodeRoutesStatuses[nodeName] = make(map[string]bool)
 		l.Unlock()
 		// for every node, for every cidr
 		for _, podCIDR := range node.Spec.PodCIDRs {
-			// we add it to our nodeCIDRs map here because if we don't consider Node addresses change,
-			// add and delete go routines run simultaneously.
+			// we add it to our nodeCIDRs map here because add and delete go routines run at the same time
 			l.Lock()
-			action := getRouteAction(rn.routes, podCIDR, nodeName, node.Status.Addresses)
-			(*routeMap[nodeName].cidrWithActions)[podCIDR] = action
+			nodeRoutesStatuses[nodeName][podCIDR] = false
 			l.Unlock()
-			klog.Infof("action for Node %q with CIDR %q: %q", nodeName, podCIDR, action)
-		}
-	}
-
-	// searches our bag of node -> cidrs for a match
-	// If the action doesn't exist, action is remove or update, then the route should be deleted.
-	shouldDeleteRoute := func(nodeName types.NodeName, cidr string) bool {
-		l.Lock()
-		defer l.Unlock()
-
-		cidrWithActions := routeMap[nodeName].cidrWithActions
-		if cidrWithActions == nil {
-			return true
-		}
-		action, exist := (*cidrWithActions)[cidr]
-		if !exist || action == remove || action == update {
-			klog.Infof("route should be deleted, spec: exist: %v, action: %q, Node %q, CIDR %q", exist, action, nodeName, cidr)
-			return true
-		}
-		return false
-	}
-
-	// remove routes that are not in use or need to be updated.
-	for _, route := range routes {
-		if !rc.isResponsibleForRoute(route) {
-			continue
-		}
-		// Check if this route is a blackhole, or applies to a node we know about & CIDR status is created.
-		if route.Blackhole || shouldDeleteRoute(route.TargetNode, route.DestinationCIDR) {
-			wg.Add(1)
-			// Delete the route.
-			go func(route *cloudprovider.Route, startTime time.Time) {
-				defer wg.Done()
-				// respect the rate limiter
-				rateLimiter <- struct{}{}
-				klog.Infof("Deleting route %s %s", route.Name, route.DestinationCIDR)
-				if err := rc.routes.DeleteRoute(ctx, rc.clusterName, route); err != nil {
-					klog.Errorf("Could not delete route %s %s after %v: %v", route.Name, route.DestinationCIDR, time.Since(startTime), err)
-				} else {
-					klog.Infof("Deleted route %s %s after %v", route.Name, route.DestinationCIDR, time.Since(startTime))
-				}
-				<-rateLimiter
-			}(route, time.Now())
-		}
-	}
-	// https://github.com/kubernetes/kubernetes/issues/98359
-	// When routesUpdated is true, Route addition and deletion cannot run simultaneously because if action is update,
-	// the same route may be added and deleted.
-	if len(routes) != 0 && routes[0].EnableNodeAddresses {
-		wg.Wait()
-	}
-
-	// Now create new routes or update existing ones.
-	for _, node := range nodes {
-		// Skip if the node hasn't been assigned a CIDR yet.
-		if len(node.Spec.PodCIDRs) == 0 {
-			continue
-		}
-		nodeName := types.NodeName(node.Name)
-
-		// for every node, for every cidr
-		for _, podCIDR := range node.Spec.PodCIDRs {
-			l.Lock()
-			action := (*routeMap[nodeName].cidrWithActions)[podCIDR]
-			l.Unlock()
-			if action == keep || action == remove {
+			// ignore if already created
+			if hasRoute(routeMap, nodeName, podCIDR) {
+				l.Lock()
+				nodeRoutesStatuses[nodeName][podCIDR] = true // a route for this podCIDR is already created
+				l.Unlock()
 				continue
 			}
 			// if we are here, then a route needs to be created for this node
 			route := &cloudprovider.Route{
-				TargetNode:          nodeName,
-				TargetNodeAddresses: node.Status.Addresses,
-				DestinationCIDR:     podCIDR,
+				TargetNode:      nodeName,
+				DestinationCIDR: podCIDR,
 			}
-			klog.Infof("route spec to be created: %v", route)
 			// cloud providers that:
 			// - depend on nameHint
 			// - trying to support dual stack
@@ -303,7 +188,7 @@ func (rc *RouteController) reconcile(ctx context.Context, nodes []*v1.Node, rout
 				defer wg.Done()
 				err := clientretry.RetryOnConflict(updateNetworkConditionBackoff, func() error {
 					startTime := time.Now()
-					// Ensure that we don't have more than maxConcurrentRouteOperations
+					// Ensure that we don't have more than maxConcurrentRouteCreations
 					// CreateRoute calls in flight.
 					rateLimiter <- struct{}{}
 					klog.Infof("Creating route for node %s %s with hint %s, throttled %v", nodeName, route.DestinationCIDR, nameHint, time.Since(startTime))
@@ -324,8 +209,7 @@ func (rc *RouteController) reconcile(ctx context.Context, nodes []*v1.Node, rout
 						}
 					}
 					l.Lock()
-					// Mark the route action as done (keep)
-					(*routeMap[nodeName].cidrWithActions)[route.DestinationCIDR] = keep
+					nodeRoutesStatuses[nodeName][route.DestinationCIDR] = true
 					l.Unlock()
 					klog.Infof("Created route for node %s %s with hint %s after %v", nodeName, route.DestinationCIDR, nameHint, time.Since(startTime))
 					return nil
@@ -336,32 +220,64 @@ func (rc *RouteController) reconcile(ctx context.Context, nodes []*v1.Node, rout
 			}(nodeName, nameHint, route)
 		}
 	}
+
+	// searches our bag of node->cidrs for a match
+	nodeHasCidr := func(nodeName types.NodeName, cidr string) bool {
+		l.Lock()
+		defer l.Unlock()
+
+		nodeRoutes := nodeRoutesStatuses[nodeName]
+		if nodeRoutes == nil {
+			return false
+		}
+		_, exist := nodeRoutes[cidr]
+		return exist
+	}
+	// delete routes that are not in use
+	for _, route := range routes {
+		if rc.isResponsibleForRoute(route) {
+			// Check if this route is a blackhole, or applies to a node we know about & has an incorrect CIDR.
+			if route.Blackhole || !nodeHasCidr(route.TargetNode, route.DestinationCIDR) {
+				wg.Add(1)
+				// Delete the route.
+				go func(route *cloudprovider.Route, startTime time.Time) {
+					defer wg.Done()
+					// respect the rate limiter
+					rateLimiter <- struct{}{}
+					klog.Infof("Deleting route %s %s", route.Name, route.DestinationCIDR)
+					if err := rc.routes.DeleteRoute(ctx, rc.clusterName, route); err != nil {
+						klog.Errorf("Could not delete route %s %s after %v: %v", route.Name, route.DestinationCIDR, time.Since(startTime), err)
+					} else {
+						klog.Infof("Deleted route %s %s after %v", route.Name, route.DestinationCIDR, time.Since(startTime))
+					}
+					<-rateLimiter
+				}(route, time.Now())
+			}
+		}
+	}
 	wg.Wait()
 
-	// after all route actions have been done (or not), we start updating
+	// after all routes have been created (or not), we start updating
 	// all nodes' statuses with the outcome
 	for _, node := range nodes {
-		actions := routeMap[types.NodeName(node.Name)].cidrWithActions
-		if actions == nil {
-			continue
-		}
-
 		wg.Add(1)
-		if len(*actions) == 0 {
+		nodeRoutes := nodeRoutesStatuses[types.NodeName(node.Name)]
+		allRoutesCreated := true
+
+		if len(nodeRoutes) == 0 {
 			go func(n *v1.Node) {
 				defer wg.Done()
 				klog.Infof("node %v has no routes assigned to it. NodeNetworkUnavailable will be set to true", n.Name)
 				if err := rc.updateNetworkingCondition(n, false); err != nil {
-					klog.Errorf("failed to update networking condition when no actions: %v", err)
+					klog.Errorf("failed to update networking condition when no nodeRoutes: %v", err)
 				}
 			}(node)
 			continue
 		}
 
-		// check if all route actions were done. if so, then it should be ready
-		allRoutesCreated := true
-		for _, action := range *actions {
-			if action == add || action == update {
+		// check if all routes were created. if so, then it should be ready
+		for _, created := range nodeRoutes {
+			if !created {
 				allRoutesCreated = false
 				break
 			}
@@ -449,35 +365,14 @@ func (rc *RouteController) isResponsibleForRoute(route *cloudprovider.Route) boo
 	return false
 }
 
-// getRouteAction returns an action according to if there's a route matches a specific cidr and target Node addresses.
-func getRouteAction(routes []*cloudprovider.Route, cidr string, nodeName types.NodeName, realNodeAddrs []v1.NodeAddress) routeAction {
-	for _, route := range routes {
-		if route.DestinationCIDR == cidr {
-			if !route.EnableNodeAddresses || equalNodeAddrs(realNodeAddrs, route.TargetNodeAddresses) {
-				return keep
-			}
-			klog.Infof("Node addresses have changed from %v to %v", route.TargetNodeAddresses, realNodeAddrs)
-			return update
-		}
-	}
-	return add
-}
-
-func equalNodeAddrs(addrs0 []v1.NodeAddress, addrs1 []v1.NodeAddress) bool {
-	if len(addrs0) != len(addrs1) {
-		return false
-	}
-	for _, ip0 := range addrs0 {
-		found := false
-		for _, ip1 := range addrs1 {
-			if reflect.DeepEqual(ip0, ip1) {
-				found = true
-				break
+// checks if a node owns a route with a specific cidr
+func hasRoute(rm map[types.NodeName][]*cloudprovider.Route, nodeName types.NodeName, cidr string) bool {
+	if routes, ok := rm[nodeName]; ok {
+		for _, route := range routes {
+			if route.DestinationCIDR == cidr {
+				return true
 			}
 		}
-		if !found {
-			return false
-		}
 	}
-	return true
+	return false
 }
