@@ -19,11 +19,9 @@ package daemonset
 import (
 	"context"
 	"fmt"
-	"strings"
 	"testing"
 	"time"
 
-	"github.com/google/go-cmp/cmp"
 	apps "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
@@ -41,12 +39,10 @@ import (
 	"k8s.io/client-go/tools/events"
 	"k8s.io/client-go/util/flowcontrol"
 	"k8s.io/client-go/util/retry"
-	"k8s.io/kubernetes/cmd/kube-apiserver/app/options"
 	kubeapiservertesting "k8s.io/kubernetes/cmd/kube-apiserver/app/testing"
 	podutil "k8s.io/kubernetes/pkg/api/v1/pod"
 	"k8s.io/kubernetes/pkg/controller"
 	"k8s.io/kubernetes/pkg/controller/daemon"
-	"k8s.io/kubernetes/pkg/controlplane"
 	"k8s.io/kubernetes/pkg/scheduler"
 	"k8s.io/kubernetes/pkg/scheduler/profile"
 	labelsutil "k8s.io/kubernetes/pkg/util/labels"
@@ -56,26 +52,14 @@ import (
 var zero = int64(0)
 
 func setup(t *testing.T) (context.Context, kubeapiservertesting.TearDownFunc, *daemon.DaemonSetsController, informers.SharedInformerFactory, clientset.Interface) {
-	return setupWithServerSetup(t, framework.TestServerSetup{})
-}
+	// Disable ServiceAccount admission plugin as we don't have serviceaccount controller running.
+	server := kubeapiservertesting.StartTestServerOrDie(t, nil, []string{"--disable-admission-plugins=ServiceAccount,TaintNodesByCondition"}, framework.SharedEtcd())
 
-func setupWithServerSetup(t *testing.T, serverSetup framework.TestServerSetup) (context.Context, kubeapiservertesting.TearDownFunc, *daemon.DaemonSetsController, informers.SharedInformerFactory, clientset.Interface) {
-	modifyServerRunOptions := serverSetup.ModifyServerRunOptions
-	serverSetup.ModifyServerRunOptions = func(opts *options.ServerRunOptions) {
-		if modifyServerRunOptions != nil {
-			modifyServerRunOptions(opts)
-		}
-
-		opts.Admission.GenericAdmission.DisablePlugins = append(opts.Admission.GenericAdmission.DisablePlugins,
-			// Disable ServiceAccount admission plugin as we don't have
-			// serviceaccount controller running.
-			"ServiceAccount",
-			"TaintNodesByCondition",
-		)
+	config := restclient.CopyConfig(server.ClientConfig)
+	clientSet, err := clientset.NewForConfig(config)
+	if err != nil {
+		t.Fatalf("Error in creating clientset: %v", err)
 	}
-
-	clientSet, config, closeFn := framework.StartTestServer(t, serverSetup)
-
 	resyncPeriod := 12 * time.Hour
 	informers := informers.NewSharedInformerFactory(clientset.NewForConfigOrDie(restclient.AddUserAgent(config, "daemonset-informers")), resyncPeriod)
 	dc, err := daemon.NewDaemonSetsController(
@@ -112,7 +96,7 @@ func setupWithServerSetup(t *testing.T, serverSetup framework.TestServerSetup) (
 
 	tearDownFn := func() {
 		cancel()
-		closeFn()
+		server.TearDownFn()
 		eventBroadcaster.Shutdown()
 	}
 
@@ -274,7 +258,9 @@ func validateDaemonSetPodsAndMarkReady(
 ) {
 	if err := wait.Poll(time.Second, 60*time.Second, func() (bool, error) {
 		objects := podInformer.GetIndexer().List()
-		nonTerminatedPods := 0
+		if len(objects) != numberPods {
+			return false, nil
+		}
 
 		for _, object := range objects {
 			pod := object.(*v1.Pod)
@@ -291,10 +277,6 @@ func validateDaemonSetPodsAndMarkReady(
 				t.Errorf("controllerRef.Controller is not set to true")
 			}
 
-			if podutil.IsPodPhaseTerminal(pod.Status.Phase) {
-				continue
-			}
-			nonTerminatedPods++
 			if !podutil.IsPodReady(pod) && len(pod.Spec.NodeName) != 0 {
 				podCopy := pod.DeepCopy()
 				podCopy.Status = v1.PodStatus{
@@ -308,80 +290,12 @@ func validateDaemonSetPodsAndMarkReady(
 			}
 		}
 
-		return nonTerminatedPods == numberPods, nil
+		return true, nil
 	}); err != nil {
 		t.Fatal(err)
 	}
 }
 
-func validateDaemonSetPodsActive(
-	podClient corev1client.PodInterface,
-	podInformer cache.SharedIndexInformer,
-	numberPods int,
-	t *testing.T,
-) {
-	if err := wait.Poll(time.Second, 60*time.Second, func() (bool, error) {
-		objects := podInformer.GetIndexer().List()
-		if len(objects) < numberPods {
-			return false, nil
-		}
-		podsActiveCount := 0
-		for _, object := range objects {
-			pod := object.(*v1.Pod)
-			ownerReferences := pod.ObjectMeta.OwnerReferences
-			if len(ownerReferences) != 1 {
-				return false, fmt.Errorf("Pod %s has %d OwnerReferences, expected only 1", pod.Name, len(ownerReferences))
-			}
-			controllerRef := ownerReferences[0]
-			if got, want := controllerRef.Kind, "DaemonSet"; got != want {
-				t.Errorf("controllerRef.Kind = %q, want %q", got, want)
-			}
-			if controllerRef.Controller == nil || *controllerRef.Controller != true {
-				t.Errorf("controllerRef.Controller is not set to true")
-			}
-			if pod.Status.Phase == v1.PodRunning || pod.Status.Phase == v1.PodPending {
-				podsActiveCount += 1
-			}
-		}
-		return podsActiveCount == numberPods, nil
-	}); err != nil {
-		t.Fatal(err)
-	}
-}
-
-func validateDaemonSetPodsTolerations(
-	podClient corev1client.PodInterface,
-	podInformer cache.SharedIndexInformer,
-	expectedTolerations []v1.Toleration,
-	prefix string,
-	t *testing.T,
-) {
-	objects := podInformer.GetIndexer().List()
-	for _, object := range objects {
-		var prefixedPodToleration []v1.Toleration
-		pod := object.(*v1.Pod)
-		ownerReferences := pod.ObjectMeta.OwnerReferences
-		if len(ownerReferences) != 1 {
-			t.Errorf("Pod %s has %d OwnerReferences, expected only 1", pod.Name, len(ownerReferences))
-		}
-		controllerRef := ownerReferences[0]
-		if got, want := controllerRef.Kind, "DaemonSet"; got != want {
-			t.Errorf("controllerRef.Kind = %q, want %q", got, want)
-		}
-		if controllerRef.Controller == nil || *controllerRef.Controller != true {
-			t.Errorf("controllerRef.Controller is not set to true")
-		}
-		for _, podToleration := range pod.Spec.Tolerations {
-			if strings.HasPrefix(podToleration.Key, prefix) {
-				prefixedPodToleration = append(prefixedPodToleration, podToleration)
-			}
-		}
-		if diff := cmp.Diff(expectedTolerations, prefixedPodToleration); diff != "" {
-			t.Fatalf("Unexpected tolerations (-want,+got):\n%s", diff)
-		}
-	}
-}
-
 // podUnschedulable returns a condition function that returns true if the given pod
 // gets unschedulable status.
 func podUnschedulable(c clientset.Interface, podNamespace, podName string) wait.ConditionFunc {
@@ -482,23 +396,6 @@ func validateDaemonSetStatus(
 	}
 }
 
-func validateUpdatedNumberScheduled(
-	ctx context.Context,
-	dsClient appstyped.DaemonSetInterface,
-	dsName string,
-	expectedUpdatedNumberScheduled int32,
-	t *testing.T) {
-	if err := wait.Poll(time.Second, 60*time.Second, func() (bool, error) {
-		ds, err := dsClient.Get(context.TODO(), dsName, metav1.GetOptions{})
-		if err != nil {
-			return false, err
-		}
-		return ds.Status.UpdatedNumberScheduled == expectedUpdatedNumberScheduled, nil
-	}); err != nil {
-		t.Fatal(err)
-	}
-}
-
 func updateDS(t *testing.T, dsClient appstyped.DaemonSetInterface, dsName string, updateFunc func(*apps.DaemonSet)) *apps.DaemonSet {
 	var ds *apps.DaemonSet
 	if err := retry.RetryOnConflict(retry.DefaultBackoff, func() error {
@@ -953,23 +850,30 @@ func TestDSCUpdatesPodLabelAfterDedupCurHistories(t *testing.T) {
 		t.Fatalf("Failed to update the pod label after new controllerrevision is created: %v", err)
 	}
 
-	revs, err := clientset.AppsV1().ControllerRevisions(ds.Namespace).List(context.TODO(), metav1.ListOptions{})
-	if err != nil {
-		t.Fatalf("Failed to list controllerrevision: %v", err)
-	}
-	if revs.Size() == 0 {
-		t.Fatalf("No avaialable controllerrevision")
-	}
+	err = wait.PollImmediate(1*time.Second, 10*time.Second, func() (bool, error) {
+		revs, err := clientset.AppsV1().ControllerRevisions(ds.Namespace).List(context.TODO(), metav1.ListOptions{})
+		if err != nil {
+			return false, fmt.Errorf("failed to list controllerrevision: %v", err)
+		}
+		if revs.Size() == 0 {
+			return false, fmt.Errorf("no avaialable controllerrevision")
+		}
 
-	for _, rev := range revs.Items {
-		t.Logf("revision: %v;hash: %v", rev.Name, rev.ObjectMeta.Labels[apps.DefaultDaemonSetUniqueLabelKey])
-		for _, oref := range rev.OwnerReferences {
-			if oref.Kind == "DaemonSet" && oref.UID == ds.UID {
-				if rev.Name != newName {
-					t.Fatalf("duplicate controllerrevision is not deleted")
+		for _, rev := range revs.Items {
+			t.Logf("revision: %v;hash: %v", rev.Name, rev.ObjectMeta.Labels[apps.DefaultDaemonSetUniqueLabelKey])
+			for _, oref := range rev.OwnerReferences {
+				if oref.Kind == "DaemonSet" && oref.UID == ds.UID {
+					if rev.Name != newName {
+						t.Logf("waiting for duplicate controllerrevision %v to be deleted", newName)
+						return false, nil
+					}
 				}
 			}
 		}
+		return true, nil
+	})
+	if err != nil {
+		t.Fatalf("Failed to check that duplicate controllerrevision is not deleted: %v", err)
 	}
 }
 
@@ -1095,158 +999,3 @@ func TestUnschedulableNodeDaemonDoesLaunchPod(t *testing.T) {
 		validateDaemonSetStatus(dsClient, ds.Name, 2, t)
 	})
 }
-
-func TestUpdateStatusDespitePodCreationFailure(t *testing.T) {
-	forEachStrategy(t, func(t *testing.T, strategy *apps.DaemonSetUpdateStrategy) {
-		limitedPodNumber := 2
-		ctx, closeFn, dc, informers, clientset := setupWithServerSetup(t, framework.TestServerSetup{
-			ModifyServerConfig: func(config *controlplane.Config) {
-				config.GenericConfig.AdmissionControl = &fakePodFailAdmission{
-					limitedPodNumber: limitedPodNumber,
-				}
-			},
-		})
-		defer closeFn()
-		ns := framework.CreateNamespaceOrDie(clientset, "update-status-despite-pod-failure", t)
-		defer framework.DeleteNamespaceOrDie(clientset, ns, t)
-
-		dsClient := clientset.AppsV1().DaemonSets(ns.Name)
-		podClient := clientset.CoreV1().Pods(ns.Name)
-		nodeClient := clientset.CoreV1().Nodes()
-		podInformer := informers.Core().V1().Pods().Informer()
-
-		informers.Start(ctx.Done())
-		go dc.Run(ctx, 2)
-
-		ds := newDaemonSet("foo", ns.Name)
-		ds.Spec.UpdateStrategy = *strategy
-		_, err := dsClient.Create(context.TODO(), ds, metav1.CreateOptions{})
-		if err != nil {
-			t.Fatalf("Failed to create DaemonSet: %v", err)
-		}
-		defer cleanupDaemonSets(t, clientset, ds)
-
-		addNodes(nodeClient, 0, 5, nil, t)
-
-		validateDaemonSetPodsAndMarkReady(podClient, podInformer, limitedPodNumber, t)
-		validateDaemonSetStatus(dsClient, ds.Name, int32(limitedPodNumber), t)
-	})
-}
-
-func TestDaemonSetRollingUpdateWithTolerations(t *testing.T) {
-	var taints []v1.Taint
-	var node *v1.Node
-	var tolerations []v1.Toleration
-	ctx, closeFn, dc, informers, clientset := setup(t)
-	defer closeFn()
-	ns := framework.CreateNamespaceOrDie(clientset, "daemonset-rollingupdate-with-tolerations-test", t)
-	defer framework.DeleteNamespaceOrDie(clientset, ns, t)
-
-	dsClient := clientset.AppsV1().DaemonSets(ns.Name)
-	podClient := clientset.CoreV1().Pods(ns.Name)
-	nodeClient := clientset.CoreV1().Nodes()
-	podInformer := informers.Core().V1().Pods().Informer()
-	informers.Start(ctx.Done())
-	go dc.Run(ctx, 2)
-
-	zero := intstr.IntOrString{Type: intstr.Int, IntVal: 0}
-	maxSurge := intstr.IntOrString{Type: intstr.Int, IntVal: 1}
-	ds := newDaemonSet("foo", ns.Name)
-	ds.Spec.UpdateStrategy = apps.DaemonSetUpdateStrategy{
-		Type: apps.RollingUpdateDaemonSetStrategyType,
-		RollingUpdate: &apps.RollingUpdateDaemonSet{
-			MaxUnavailable: &zero,
-			MaxSurge:       &maxSurge,
-		},
-	}
-
-	// Add six nodes with zone-y, zone-z or common taint
-	for i := 0; i < 6; i++ {
-		if i < 2 {
-			taints = []v1.Taint{
-				{Key: "zone-y", Effect: v1.TaintEffectNoSchedule},
-			}
-		} else if i < 4 {
-			taints = []v1.Taint{
-				{Key: "zone-z", Effect: v1.TaintEffectNoSchedule},
-			}
-		} else {
-			taints = []v1.Taint{
-				{Key: "zone-common", Effect: v1.TaintEffectNoSchedule},
-			}
-		}
-		node = newNode(fmt.Sprintf("node-%d", i), nil)
-		node.Spec.Taints = taints
-		_, err := nodeClient.Create(context.TODO(), node, metav1.CreateOptions{})
-		if err != nil {
-			t.Fatalf("Failed to create node: %v", err)
-		}
-	}
-
-	// Create DaemonSet with zone-y toleration
-	tolerations = []v1.Toleration{
-		{Key: "zone-y", Operator: v1.TolerationOpExists},
-		{Key: "zone-common", Operator: v1.TolerationOpExists},
-	}
-	ds.Spec.Template.Spec.Tolerations = tolerations
-	_, err := dsClient.Create(ctx, ds, metav1.CreateOptions{})
-	if err != nil {
-		t.Fatalf("Failed to create DaemonSet: %v", err)
-	}
-	defer cleanupDaemonSets(t, clientset, ds)
-	validateDaemonSetPodsActive(podClient, podInformer, 4, t)
-	validateDaemonSetPodsAndMarkReady(podClient, podInformer, 4, t)
-	validateDaemonSetStatus(dsClient, ds.Name, 4, t)
-	validateUpdatedNumberScheduled(ctx, dsClient, ds.Name, 4, t)
-	validateDaemonSetPodsTolerations(podClient, podInformer, tolerations, "zone-", t)
-
-	// Update DaemonSet with zone-z toleration
-	tolerations = []v1.Toleration{
-		{Key: "zone-z", Operator: v1.TolerationOpExists},
-		{Key: "zone-common", Operator: v1.TolerationOpExists},
-	}
-	ds.Spec.Template.Spec.Tolerations = tolerations
-	_, err = dsClient.Update(ctx, ds, metav1.UpdateOptions{})
-	if err != nil {
-		t.Fatalf("Failed to update DaemonSet: %v", err)
-	}
-
-	// Expected numberPods of validateDaemonSetPodsActive is 7 when update DaemonSet
-	// and before updated pods become ready because:
-	//   - New 2 pods are created and Pending on Zone Z nodes
-	//   - New 1 pod are created as surge and Pending on Zone Common node
-	//   - Old 2 pods that violate scheduling constraints on Zone Y nodes will remain existing and Running
-	//     until other new pods become available
-	validateDaemonSetPodsActive(podClient, podInformer, 7, t)
-	validateDaemonSetPodsAndMarkReady(podClient, podInformer, 4, t)
-	validateDaemonSetStatus(dsClient, ds.Name, 4, t)
-	validateUpdatedNumberScheduled(ctx, dsClient, ds.Name, 4, t)
-	validateDaemonSetPodsTolerations(podClient, podInformer, tolerations, "zone-", t)
-
-	// Update DaemonSet with zone-y and zone-z toleration
-	tolerations = []v1.Toleration{
-		{Key: "zone-y", Operator: v1.TolerationOpExists},
-		{Key: "zone-z", Operator: v1.TolerationOpExists},
-		{Key: "zone-common", Operator: v1.TolerationOpExists},
-	}
-	ds.Spec.Template.Spec.Tolerations = tolerations
-	_, err = dsClient.Update(ctx, ds, metav1.UpdateOptions{})
-	if err != nil {
-		t.Fatalf("Failed to update DaemonSet: %v", err)
-	}
-	validateDaemonSetPodsActive(podClient, podInformer, 7, t)
-	validateDaemonSetPodsAndMarkReady(podClient, podInformer, 6, t)
-	validateDaemonSetStatus(dsClient, ds.Name, 6, t)
-	validateUpdatedNumberScheduled(ctx, dsClient, ds.Name, 6, t)
-	validateDaemonSetPodsTolerations(podClient, podInformer, tolerations, "zone-", t)
-
-	// Update DaemonSet with no toleration
-	ds.Spec.Template.Spec.Tolerations = nil
-	_, err = dsClient.Update(ctx, ds, metav1.UpdateOptions{})
-	if err != nil {
-		t.Fatalf("Failed to update DaemonSet: %v", err)
-	}
-	validateDaemonSetPodsActive(podClient, podInformer, 0, t)
-	validateDaemonSetStatus(dsClient, ds.Name, 0, t)
-	validateUpdatedNumberScheduled(ctx, dsClient, ds.Name, 0, t)
-}
